==> muse-0.0.1.tar.gz: muse/jmixer.cpp <==
/* jmixer.cpp
 * (c)2000 Denis Roio aka jaromil
 * based on the eMixer code by Dunstan B. Nesbit
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/soundcard.h>
#include <signal.h>
#include <sys/types.h>
#include <termios.h>
#include <sys/time.h>

extern "C" {
#include "shout.h"
#include "mpeg.h"
}

#define BUF_SIZE 2048
#define MAX_SAMPLE 32766
#define CLIP 30000
#define CLIP_A ((MAX_SAMPLE-CLIP)*(MAX_SAMPLE-CLIP)) // 7650756
#define CLIP_B (MAX_SAMPLE-2*CLIP) // 982980000
#define MAX_CHANNELS 8
#define MAX_PLAYLIST_ENTRY 100
#define MAX_PATH_SIZE 240

// struct termios tsave;

struct channel {
  short inbuf[BUF_SIZE];
  char *name;
  float volume;
  float speed;
  int rate;
  int channels;
  bool on;
  int end;
  int stayon;
  int stream;
  FILE *fstr;
  pid_t pid;
  int chindx;
};

class Stream_mixer {

private:

  char playlist[MAX_PLAYLIST_ENTRY][MAX_PATH_SIZE];

  short audio_buffer[BUF_SIZE];
  int process_buffer[BUF_SIZE];
  short work1[BUF_SIZE*40], work2[BUF_SIZE];
  short inbuf[BUF_SIZE*40];

  int dsp;
  shout_conn_t ice_conn;
  enum { DSP, ICECAST } output;
  
  /* conf & m_dat */
  int sample_rate;
  int stereo;
  int c_number;
  int rate;
  int channels;
  channel stream_channel[MAX_CHANNELS];
  
  bool write_toicecast(short *sendbuf, long len);
  void close_soundcard();

  void get_audio(int ch);
  int spawn_stream(int ch);
  void kill_channel(int ch);

  void make_mono(short *stereo, short *mono, int num);
  void make_stereo(short *mono, short *stereo, int num);
  void sep_stereo(short *stereo, short *left, short *right, int num);
  void mix_stereo(short *left, short *right, short *stereo, int num);

  void int_compress_samples(short *in, short *out, int num, int grain);
  void int_stretch_samples(short *in, short *out, int num, int grain);

  void compress_samples(short *in, short *out,unsigned int num1,unsigned int num2);
  void stretch_samples(short *in, short *out,unsigned int num1,unsigned int num2);
  void change_time(short *in, short *out, unsigned int num_in, unsigned int num_out);
  void convert_audio(short *in,short *out,int ch1,int ch2,int n1,int n2);
  
public:
  Stream_mixer(char *pl);
  
  bool open_soundcard();
  bool open_icecast(char *ip, int port, char *pass);
  void play_chunks();
  void start_channel(int ch);
  void stop_channel(int ch);
  void set_track(int ch, int playlist_pos);
  void set_volume(int ch, float vol);
  void set_speed(int ch, float speed);

  ~Stream_mixer();
};

Stream_mixer::Stream_mixer(char *pl) {
  /* fills Stream_mixer::playlist[][] from file pointed by *pl pathname */
  FILE *plist;
  int c = 0;
  char *p;

  plist = fopen(pl,"r");
  if(plist == NULL)
    fprintf(stderr,"no playlist\n");

  while (fgets(&playlist[c][0],MAX_PATH_SIZE,plist) != NULL) {
    p = &playlist[c][0];
    while(*p!='\n') p++;
    *p = '\0';
    c++;
  }
  
  sample_rate=44100;
  stereo=1;
  
  /* compatibility - will go away */
  c_number=0;
  rate=sample_rate;
  channels=stereo+1;

  /* init channels */
  for(c=0;c<MAX_CHANNELS;c++) {
    stream_channel[c].volume=1.0;
    stream_channel[c].speed=1.0;
    stream_channel[c].rate=0;
    stream_channel[c].channels=0;
    stream_channel[c].on=0;
    stream_channel[c].end=0;
    stream_channel[c].stayon=1;
    c_number++;
  }
}

Stream_mixer::~Stream_mixer() {
  int i;

  for(i=0;i<MAX_CHANNELS;i++)
    kill_channel(i);

  switch(output) {
  case DSP:
    close_soundcard();
    break;
  case ICECAST:
    shout_disconnect(&ice_conn);
    break;
  default:
    break;
  }
}

bool Stream_mixer::open_soundcard()
{
    int format,tstereo,speed;
    int card;

    if((dsp=open("/dev/dsp",O_WRONLY))==-1) {
        perror("Error opening /dev/dsp: ");
	return(false);
    } else
      output = DSP;

    fprintf(stderr,"Soundcard output initialized\n");

    format = AFMT_S16_LE;
    tstereo = stereo;
    speed = sample_rate;

    ioctl(dsp, SNDCTL_DSP_SETDUPLEX, 0);
    ioctl(dsp, DSP_CAP_DUPLEX, 0);

    if(ioctl(dsp,SNDCTL_DSP_SETFMT,&format)==-1)
      fprintf(stderr,"Soundcard: Failed to set data format\n");
    if(ioctl(dsp,SNDCTL_DSP_STEREO,&tstereo)==-1)
      fprintf(stderr,"Souncard: Something went wrong with the stereo setting\n");
    if(ioctl(dsp,SNDCTL_DSP_SPEED,&speed)==-1)
      fprintf(stderr,"Soundcard: Speed Setting failed\n");
    return(true);
} /* open_soundcard */

bool Stream_mixer::open_icecast(char *ip, int port, char *pass) {
  shout_init_connection(&ice_conn);
  ice_conn.ip = ip;
  ice_conn.port = port;
  ice_conn.password = pass;
  
  if(shout_connect(&ice_conn)) {
    fprintf(stderr,"Connected to server %s\n",ip);
    output = ICECAST;
    return(true);
  } else {
    fprintf(stderr,"Unable to connect to %s\n",ip);
    return(false);
  }
}

void Stream_mixer::play_chunks()
{
  int i,j,num;
  
  num=BUF_SIZE;
  
  /* here would be better a memset */
  for(j=0;j<num;j++)process_buffer[j]=0;
  
  for(i=0;i<MAX_CHANNELS;i++)
    {
      //      if(stream_channel[i].on!=NULL)
      //        {
      if(stream_channel[i].on)
	{
	  //mixer.c_list[i]->end=
	  //	      printf("playing channel %d\n",i);
	  get_audio(i);
	  //if(mixer.c_list[i]->end)printf("TRACK END***************");
	  /* if the correct output mode is selected then play this channel */
	      
	  for(j=0;j<num;j++)
	    /* il casting e' per via di un warning... funzia? */
	    process_buffer[j] = (int) (process_buffer[j] + (stream_channel[i].inbuf[j] * stream_channel[i].volume));
	}
      //        }
    }
  
  for(j=0;j<num;j++)
    {
      if((process_buffer[j]<CLIP)&&(process_buffer[j]>-CLIP))
	audio_buffer[j]=(short)(process_buffer[j]);
      else if (process_buffer[j]>0)
	audio_buffer[j]=MAX_SAMPLE-(CLIP_A/(CLIP_B+process_buffer[j]));
      else
	audio_buffer[j]=-(MAX_SAMPLE-(CLIP_A/(CLIP_B-process_buffer[j])));
    }
  
  /* send output to icecast */
  /* this is incorrect - should be passed to the mp3 encoder */
  switch(output) {
  case DSP:
    write(dsp,audio_buffer,num<<1);
    break;
  case ICECAST:
    write_toicecast(audio_buffer,num<<1);
    break;
  default:
    break;
  }
  // write(audio_fd,audio_buffer,num*2); 
  
  /*max=0;
    for(i=0;i<BUF_SIZE;i++)
    {
    if(abs(audio_buffer[0][i])>max)
    max=abs(audio_buffer[0][i]);
    }  */
}

void Stream_mixer::start_channel(int ch) { stream_channel[ch].on = true; }

void Stream_mixer::stop_channel(int ch) { stream_channel[ch].on = false; }

void Stream_mixer::set_track(int ch, int playlist_pos) {
  kill_channel(ch);
  stream_channel[ch].name = &playlist[playlist_pos][0];
  stream_channel[ch].on=true;
  stream_channel[ch].end=0;
  spawn_stream(ch);
}

void Stream_mixer::set_volume(int ch, float vol) { stream_channel[ch].volume = vol; }

void Stream_mixer::set_speed(int ch, float speed) { stream_channel[ch].speed = speed; }

void Stream_mixer::kill_channel(int ch) {
    int i;
    if( stream_channel[ch].pid > 0)
    {
        kill(stream_channel[ch].pid,15);
        stream_channel[ch].pid=0;
        stream_channel[ch].on=false;
        if(stream_channel[ch].stream > 2)
        {
            fclose(stream_channel[ch].fstr);
            stream_channel[ch].stream=0;
        }
    }
    else
    {
        stream_channel[ch].pid=0;
        stream_channel[ch].on=0;
    }

    /* it would be better a memset here */
    for(i=0;i<BUF_SIZE;i++) stream_channel[ch].inbuf[i]=0;

} /* kill_channel */

bool Stream_mixer::write_toicecast(short *sendbuf, long len) {
  int res;
  res = shout_send_data(&ice_conn, (unsigned char *) sendbuf,len);
  if(!res)
    fprintf(stderr,"Send error: %i\n",ice_conn.error);
  shout_sleep(&ice_conn);
  return(res);
}

void Stream_mixer::close_soundcard() {
  ioctl(dsp, SNDCTL_DSP_RESET, 0);
  close(dsp);
}

void Stream_mixer::get_audio(int ch) {
  int i;
  unsigned int num_samples;
  int opt;

    //end=&ch->end;
    /* try starting up the decoder stream */
    if(stream_channel[ch].on)
        if(!stream_channel[ch].pid)
            if (spawn_stream(ch)) stream_channel[ch].pid=0;

    if((stream_channel[ch].on)&&(stream_channel[ch].pid))
    {
      //        num_samples=(int)((BUF_SIZE*(*ch)->rate*(*ch)->speed)/(mixer.rate*mixer.channels));
      num_samples=(int)(((stream_channel[ch].rate<<11)*stream_channel[ch].speed)/(rate*channels));
      opt = num_samples*stream_channel[ch].channels; /* QUA! */
      i=fread(inbuf,2,opt,stream_channel[ch].fstr);
        /* if we reach  the end of the stream then pad */
        if(i<opt)
        {
            while(i<opt)
            {
                inbuf[i]=0;
                i++;
            }
            stream_channel[ch].on=0;
            stream_channel[ch].end=1;

            //if(ch->stayon) ch->on=1;
        }
        /* now convert the audio into something useful */
        convert_audio(inbuf,stream_channel[ch].inbuf,stream_channel[ch].channels,channels,num_samples,BUF_SIZE/channels);
    }
    else
        for(i=0;i<BUF_SIZE;i++) stream_channel[ch].inbuf[i]=0;
    //return end;
} /* get_audio */

int Stream_mixer::spawn_stream(int ch)
{
  int strm[2],trate,tchannels;
  pid_t pid;
  
  //    j=conf.num_sndcards;
  trate=format_of(stream_channel[ch].name,&tchannels);
  stream_channel[ch].rate=trate;
  stream_channel[ch].channels=tchannels;
  
  /* create pipe */
  pipe(strm);
  pid=fork();
  if(pid)
    {
      stream_channel[ch].pid=pid;
      close(strm[1]);
      stream_channel[ch].stream=strm[0];
      stream_channel[ch].fstr=fdopen(stream_channel[ch].stream,"rb");
    }
  else
    {
      close(strm[0]);
      close(1);
      dup(strm[1]);
      close(strm[0]);
      close(strm[1]);
      /* close audio too - don't use the close_soundcard because it causes glitches*/
      // close(audio_fd);
      
      execlp("mpg123","mpg123","-sq"/*,"-n 300"*/,stream_channel[ch].name,NULL);
    }
  return 0;
} /* spawn_stream */

void Stream_mixer::make_mono(short *stereo, short *mono, int num)
{
    unsigned int i;
    for(i=0;i<num;i++)
        mono[i]=(short)(((int)stereo[i<<1]+(int)stereo[(i<<1)+1])>>1);
} /* make_mono */

void Stream_mixer::make_stereo(short *mono, short *stereo, int num)
{
    unsigned int i;
    for(i=0;i<num;i++)
        stereo[1<<1]=stereo[(i<<1)+1]=mono[i];
}/* make_stereo */

void Stream_mixer::sep_stereo(short *stereo, short *left, short *right, int num)
{
    unsigned int i;
    for(i=0;i<num;i++)
    {
       	left[i]=stereo[i<<1];
       	right[i]=stereo[(i<<1)+1];
    }
} /* sep_stereo  */

void Stream_mixer::mix_stereo(short *left, short *right, short *stereo, int num)
{
    unsigned int i;
    for(i=0;i<num;i++)
    {
        stereo[i<<1]=left[i];
	stereo[(i<<1)+1]=right[i];
    }
} /* mix_stereo */

void Stream_mixer::int_compress_samples(short *in, short *out,int num,int grain)
{
    unsigned int i,j;
    int smpl;
	
    for(i=0;i<num;i++)
    {
       	smpl=in[i*grain];
	for(j=1;j<grain;j++) smpl+=in[i*grain+j];
	out[i]=(short)(smpl/grain);

    }
} /* int_compress_samples */

void Stream_mixer::int_stretch_samples(short *in,short *out,int num,int grain)
{
    unsigned int i,j;

    for(i=0;i<num;i++)
       	for(j=0;j<grain;j++)
            out[i*grain+j]=in[i];
	
} /* int_stretch_samples  */

void Stream_mixer::compress_samples(short *in, short *out,unsigned int num1,unsigned int num2)
{
    float step=(float)(num1)/num2;
    int i;

    for(i=0;i<num2;i++)
    {
      	out[i]=in[(int)(i*step)];
    }
} /* compress_samples */

void Stream_mixer::stretch_samples(short *in, short *out,unsigned int num1,unsigned int num2)
{
    float step=(float)(num1)/num2;
    int i;

    for(i=0;i<num2;i++)
    {
       	out[i]=in[(int)(i*step)];
    }
}/* stretch_samples */

/* changes the length of a buffer of samples uses fast shortcuts where possible */
void Stream_mixer::change_time(short *in, short *out, unsigned int num_in, unsigned int num_out){
    int i;
	
    if(num_in>num_out)
    {
        /* compress samples */
        if(num_in==(num_out<<1))
        {
            int_compress_samples(in,out,num_out,2);
        }
        else if( num_in==(num_out<<2))
        {
            int_compress_samples(in,out,num_out,4);
        }
        else
        {
            compress_samples(in,out,num_in,num_out);
        }
    }
    else if(num_in<num_out)
    {
        /* stretch samples */
        if((num_in<<1)==num_out)
        {
            int_stretch_samples(in,out,num_in,2);
        }
        else if ((num_in<<2)==num_out)
        {
            int_stretch_samples(in,out,num_in,4);
        }
        else
        {
            stretch_samples(in,out,num_in,num_out);
        }
    }
    else
    {
        for(i=0;i<num_out;i++)
            out[i]=in[i];
    }
} /* change_time   */

void Stream_mixer::convert_audio(short *in,short *out,int ch1,int ch2,int n1,int n2)
{
    int i;

    if((ch1==ch2) &&(n1==n2))
    {
      int gh = ch1*n1;
      for(i=0;i<gh;i++)
	out[i]=in[i]; 
        return;
    }

    if(ch2==1){
        /* output is mono */
        if(ch1==2){
            /* input is stereo */
            make_mono(in,work1,n1);
            change_time(work1,out,n1,n2);
        }
        else
        {
            /* input is mono */
            change_time(in,out,n1,n2);
        }
    }
    else
    {
        /* output is stereo */
        if(ch1==2){
            /* input is stereo */
            sep_stereo(  in,    work1, work2, n1    );
            change_time( work1, in,    n1,    n2    );
            change_time( work2, work1, n1,    n2    );
            mix_stereo(  in,    work1, out,   n2    );
        }
        else
        {
            /* input is mono */
            change_time(in,work1,n1,n2);
            make_stereo(work1,out,  n2);
        }
    }
} /* convert_audio */

==> muse-0.0.2.tar.gz: muse/jmixer.cpp <==
/* jmixer.cpp - part of MUSE
 * (c)2000 Denis Roio aka jaromil
 * c++ engine for multiple mp3 realtime stream mixing
 * based on the eMixer code by Dunstan B. Nesbit
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <sys/soundcard.h>
#include <signal.h>
#include <sys/types.h>
#include <termios.h>
#include <sys/time.h>

extern "C" {
#include "shout.h"
#include "mpeg.h"
}

#define BUF_SIZE 2048
#define MAX_SAMPLE 32766
#define CLIP 30000
#define CLIP_A 7650756 // ((MAX_SAMPLE-CLIP)*(MAX_SAMPLE-CLIP))
#define CLIP_B -27234 // (MAX_SAMPLE-2*CLIP)
#define MAX_CHANNELS 8
#define MAX_PLAYLIST_ENTRY 100
#define MAX_PATH_SIZE 240

// struct termios tsave;

struct channel {
  short inbuf[BUF_SIZE];
  char *name;
  float volume;
  float speed;
  int rate;
  int channels;
  bool on;
  int end;
  int stayon;
  int stream;
  FILE *fstr;
  pid_t pid;
  int chindx;
};

class Stream_mixer {

private:

  char playlist[MAX_PLAYLIST_ENTRY][MAX_PATH_SIZE];

  short audio_buffer[BUF_SIZE];
  int process_buffer[BUF_SIZE];
  short work1[BUF_SIZE*40], work2[BUF_SIZE];
  short inbuf[BUF_SIZE*40];

  int dsp;
  shout_conn_t ice_conn;
  enum { DSP, ICECAST } output;
  
  /* conf & m_dat */
  int sample_rate;
  int stereo;
  int c_number;
  int rate;
  int channels;
  channel stream_channel[MAX_CHANNELS];
  
  bool write_toicecast(short *sendbuf, long len);
  void close_soundcard();

  void get_audio(int ch);
  int spawn_stream(int ch);
  void kill_channel(int ch);

  void make_mono(short *stereo, short *mono, unsigned int num);
  void make_stereo(short *mono, short *stereo, unsigned int num);
  void sep_stereo(short *stereo, short *left, short *right, unsigned int num);
  void mix_stereo(short *left, short *right, short *stereo, unsigned int num);

  void int_compress_samples(short *in, short *out, unsigned int num, unsigned int grain);
  void int_stretch_samples(short *in, short *out, unsigned int num, unsigned int grain);

  void compress_samples(short *in, short *out,unsigned int num1,unsigned int num2);
  void stretch_samples(short *in, short *out,unsigned int num1,unsigned int num2);
  void change_time(short *in, short *out, unsigned int num_in, unsigned int num_out);
  void convert_audio(short *in,short *out,int ch1,int ch2,int n1,int n2);
  
public:
  Stream_mixer(char *pl);
  
  bool open_soundcard();
  bool open_icecast(char *ip, int port, char *pass);
  void play_chunks();
  void start_channel(int ch);
  void stop_channel(int ch);
  void set_track(int ch, int playlist_pos);
  void set_volume(int ch, float vol);
  void set_speed(int ch, float speed);

  void set_playlist(char *pl);
  void print_playlist();

  ~Stream_mixer();
};

Stream_mixer::Stream_mixer(char *pl) {
  /* fills Stream_mixer::playlist[][] from file pointed by *pl pathname */
  FILE *plist;
  int c = 0;
  char *p;

  plist = fopen(pl,"r");
  if(plist == NULL)
    fprintf(stderr,"no playlist\n");

  while (fgets(&playlist[c][0],MAX_PATH_SIZE,plist) != NULL) {
    p = &playlist[c][0];
    while(*p!='\n') p++;
    *p = '\0';
    c++;
  }
  
  sample_rate=44100;
  stereo=1;
  
  /* compatibility - will go away */
  c_number=0;
  rate=sample_rate;
  channels=stereo+1;

  /* init channels */
  for(c=0;c<MAX_CHANNELS;c++) {
    stream_channel[c].volume=1.0;
    stream_channel[c].speed=1.0;
    stream_channel[c].rate=0;
    stream_channel[c].channels=0;
    stream_channel[c].on=0;
    stream_channel[c].end=0;
    stream_channel[c].stayon=1;
    c_number++;
  }
}

Stream_mixer::~Stream_mixer() {
  int i;

  for(i=0;i<MAX_CHANNELS;i++)
    kill_channel(i);

  switch(output) {
  case DSP:
    close_soundcard();
    break;
  case ICECAST:
    shout_disconnect(&ice_conn);
    break;
  default:
    break;
  }
}

bool Stream_mixer::open_soundcard()
{
    int format,tstereo,speed;

    if((dsp=open("/dev/dsp",O_WRONLY))==-1) {
        perror("Error opening /dev/dsp: ");
	return(false);
    } else
      output = DSP;

    fprintf(stderr,"Soundcard output initialized\n");

    format = AFMT_S16_LE;
    tstereo = stereo;
    speed = sample_rate;

    ioctl(dsp, SNDCTL_DSP_SETDUPLEX, 0);
    ioctl(dsp, DSP_CAP_DUPLEX, 0);

    if(ioctl(dsp,SNDCTL_DSP_SETFMT,&format)==-1)
      fprintf(stderr,"Soundcard: Failed to set data format\n");
    if(ioctl(dsp,SNDCTL_DSP_STEREO,&tstereo)==-1)
      fprintf(stderr,"Souncard: Something went wrong with the stereo setting\n");
    if(ioctl(dsp,SNDCTL_DSP_SPEED,&speed)==-1)
      fprintf(stderr,"Soundcard: Speed Setting failed\n");
    return(true);
} /* open_soundcard */

bool Stream_mixer::open_icecast(char *ip, int port, char *pass) {
  shout_init_connection(&ice_conn);
  ice_conn.ip = ip;
  ice_conn.port = port;
  ice_conn.password = pass;
  
  if(shout_connect(&ice_conn)) {
    fprintf(stderr,"Connected to server %s\n",ip);
    output = ICECAST;
    return(true);
  } else {
    fprintf(stderr,"Unable to connect to %s\n",ip);
    return(false);
  }
}

void Stream_mixer::play_chunks()
{
  int i,j,num;
  
  num=BUF_SIZE;
  
  memset(process_buffer,0,num<<2);
  
  for(i=0;i<MAX_CHANNELS;i++) {
    if(stream_channel[i].on) {
      get_audio(i);
      if(stream_channel[i].volume!=1.0)
	for(j=0;j<num;j++)
	  process_buffer[j] = (int) (process_buffer[j] + (stream_channel[i].inbuf[j] * stream_channel[i].volume));
      else
	for(j=0;j<num;j++)
	  process_buffer[j] = (int) (process_buffer[j] + stream_channel[i].inbuf[j]);
    } // if stream_channel[i].on
  } // for i=0 to MAX_CHANNELS
  
  for(j=0;j<num;j++)
    if((process_buffer[j]<CLIP)&&(process_buffer[j]>-CLIP))
      audio_buffer[j]=(short)(process_buffer[j]);
    else if (process_buffer[j]>0)
      audio_buffer[j]=MAX_SAMPLE-(CLIP_A/(CLIP_B+process_buffer[j]));
    else
      audio_buffer[j]=-(MAX_SAMPLE-(CLIP_A/(CLIP_B-process_buffer[j])));
  
  switch(output) {
  case DSP:
    write(dsp,audio_buffer,num<<1);
    break;
  case ICECAST:
    write_toicecast(audio_buffer,num<<1);
    break;
  default:
    break;
  }
}

void Stream_mixer::start_channel(int ch) { stream_channel[ch].on = true; }

void Stream_mixer::stop_channel(int ch) { stream_channel[ch].on = false; }

void Stream_mixer::set_track(int ch, int playlist_pos) {
  kill_channel(ch);
  stream_channel[ch].name = &playlist[playlist_pos][0];
  stream_channel[ch].on=true;
  stream_channel[ch].end=0;
  spawn_stream(ch);
}

void Stream_mixer::set_volume(int ch, float vol) { stream_channel[ch].volume = vol; }

void Stream_mixer::set_speed(int ch, float speed) { stream_channel[ch].speed = speed; }

void Stream_mixer::set_playlist(char *pl) {
  FILE *plist;
  int c = 0;
  char *p;

  plist = fopen(pl,"r");
  if(plist == NULL)
    fprintf(stderr,"no playlist\n");
  
  while (fgets(&playlist[c][0],MAX_PATH_SIZE,plist) != NULL) {
    p = &playlist[c][0];
    while(*p!='\n') p++;
    *p = '\0';
    c++;
  }
}

void Stream_mixer::print_playlist() {
  int c = 0;
  fprintf(stderr,"Cached playlist:\n");
  while (playlist[c][0] != '\0') {
    fprintf(stderr,"[%02u] %s\n",c,&playlist[c][0]);
    c++;
  }
}

void Stream_mixer::kill_channel(int ch) {
    if( stream_channel[ch].pid > 0)
    {
        kill(stream_channel[ch].pid,15);
        stream_channel[ch].pid=0;
        stream_channel[ch].on=false;
        if(stream_channel[ch].stream > 2)
        {
            fclose(stream_channel[ch].fstr);
            stream_channel[ch].stream=0;
        }
    }
    else
    {
        stream_channel[ch].pid=0;
        stream_channel[ch].on=false;
    }

    /* it would be better a memset here */
    //    for(i=0;i<BUF_SIZE;i++) stream_channel[ch].inbuf[i]=0;
    memset(stream_channel[ch].inbuf,0,BUF_SIZE);

} /* kill_channel */

bool Stream_mixer::write_toicecast(short *sendbuf, long len) {
  int res;
  res = shout_send_data(&ice_conn, (unsigned char *) sendbuf,len);
  if(!res)
    fprintf(stderr,"Send error: %i\n",ice_conn.error);
  shout_sleep(&ice_conn);
  return(res);
}

void Stream_mixer::close_soundcard() {
  ioctl(dsp, SNDCTL_DSP_RESET, 0);
  close(dsp);
}

void Stream_mixer::get_audio(int ch) {
  int i;
  unsigned int num_samples;
  int opt;

    //end=&ch->end;
  /* try starting up the decoder stream 
     if(!stream_channel[ch].pid)
     if (spawn_stream(ch)) stream_channel[ch].pid=0; */

  if(stream_channel[ch].pid) {
    num_samples=(int)(((stream_channel[ch].rate<<11)*stream_channel[ch].speed)/(rate*channels));
    opt = num_samples*stream_channel[ch].channels; /* QUA! */
    i=fread(inbuf,2,opt,stream_channel[ch].fstr);
    /* if we reach  the end of the stream then pad */
    if(i<opt) {
      for(;i<opt;i++) inbuf[i]=0;
      stream_channel[ch].on=0;
      stream_channel[ch].end=1;
    }
    /* now convert the audio into something useful */
    convert_audio(inbuf,stream_channel[ch].inbuf,stream_channel[ch].channels,channels,num_samples,BUF_SIZE/channels);
  } else
    memset(stream_channel[ch].inbuf,0,BUF_SIZE);
  //return end;
} /* get_audio */

int Stream_mixer::spawn_stream(int ch)
{
  int strm[2],trate,tchannels;
  pid_t pid;
  
  //    j=conf.num_sndcards;
  trate=format_of(stream_channel[ch].name,&tchannels);
  stream_channel[ch].rate=trate;
  stream_channel[ch].channels=tchannels;
  
  /* create pipe */
  pipe(strm);
  pid=fork();
  if(pid)
    {
      stream_channel[ch].pid=pid;
      close(strm[1]);
      stream_channel[ch].stream=strm[0];
      stream_channel[ch].fstr=fdopen(stream_channel[ch].stream,"rb");
    }
  else
    {
      close(strm[0]);
      close(1);
      dup(strm[1]);
      close(strm[0]);
      close(strm[1]);
      /* close audio too - don't use the close_soundcard because it causes glitches*/
      // close(audio_fd);
      
      execlp("mpg123","mpg123","-sq"/*,"-n 300"*/,stream_channel[ch].name,NULL);
    }
  return 0;
} /* spawn_stream */

void Stream_mixer::make_mono(short *stereo, short *mono, unsigned int num)
{
    unsigned int i;
    for(i=0;i<num;i++)
        mono[i]=(short)(((int)stereo[i<<1]+(int)stereo[(i<<1)+1])>>1);
} /* make_mono */

void Stream_mixer::make_stereo(short *mono, short *stereo, unsigned int num)
{
    unsigned int i;
    for(i=0;i<num;i++)
        stereo[1<<1]=stereo[(i<<1)+1]=mono[i];
}/* make_stereo */

void Stream_mixer::sep_stereo(short *stereo, short *left, short *right, unsigned int num)
{
    unsigned int i;
    for(i=0;i<num;i++)
    {
       	left[i]=stereo[i<<1];
       	right[i]=stereo[(i<<1)+1];
    }
} /* sep_stereo  */

void Stream_mixer::mix_stereo(short *left, short *right, short *stereo, unsigned int num)
{
    unsigned int i;
    for(i=0;i<num;i++)
    {
        stereo[i<<1]=left[i];
	stereo[(i<<1)+1]=right[i];
    }
} /* mix_stereo */

void Stream_mixer::int_compress_samples(short *in, short *out,unsigned int num,unsigned int grain)
{
    unsigned int i,j;
    int smpl;
	
    for(i=0;i<num;i++)
    {
       	smpl=in[i*grain];
	for(j=1;j<grain;j++) smpl+=in[i*grain+j];
	out[i]=(short)(smpl/grain);

    }
} /* int_compress_samples */

void Stream_mixer::int_stretch_samples(short *in,short *out,unsigned int num,unsigned int grain)
{
    unsigned int i,j;

    for(i=0;i<num;i++)
       	for(j=0;j<grain;j++)
            out[i*grain+j]=in[i];
	
} /* int_stretch_samples  */

void Stream_mixer::compress_samples(short *in, short *out,unsigned int num1,unsigned int num2)
{
    float step=(float)(num1)/num2;
    unsigned int i;

    for(i=0;i<num2;i++) out[i]=in[(int)(i*step)];
} /* compress_samples */

void Stream_mixer::stretch_samples(short *in, short *out,unsigned int num1,unsigned int num2)
{
    float step=(float)(num1)/num2;
    unsigned int i;

    for(i=0;i<num2;i++) out[i]=in[(int)(i*step)];
}/* stretch_samples */

/* changes the length of a buffer of samples uses fast shortcuts where possible */
void Stream_mixer::change_time(short *in, short *out, unsigned int num_in, unsigned int num_out){
    unsigned int i;
	
    if(num_in>num_out)
    {
        /* compress samples */
        if(num_in==(num_out<<1))
        {
            int_compress_samples(in,out,num_out,2);
        }
        else if( num_in==(num_out<<2))
        {
            int_compress_samples(in,out,num_out,4);
        }
        else
        {
            compress_samples(in,out,num_in,num_out);
        }
    }
    else if(num_in<num_out)
    {
        /* stretch samples */
        if((num_in<<1)==num_out)
        {
            int_stretch_samples(in,out,num_in,2);
        }
        else if ((num_in<<2)==num_out)
        {
            int_stretch_samples(in,out,num_in,4);
        }
        else
        {
            stretch_samples(in,out,num_in,num_out);
        }
    }
    else
    {
        for(i=0;i<num_out;i++)
            out[i]=in[i];
    }
} /* change_time   */

void Stream_mixer::convert_audio(short *in,short *out,int ch1,int ch2,int n1,int n2)
{
    int i;

    if((ch1==ch2) &&(n1==n2))
    {
      int gh = ch1*n1;
      for(i=0;i<gh;i++)
	out[i]=in[i]; 
        return;
    }

    if(ch2==1){
        /* output is mono */
        if(ch1==2){
            /* input is stereo */
            make_mono(in,work1,n1);
            change_time(work1,out,n1,n2);
        }
        else
        {
            /* input is mono */
            change_time(in,out,n1,n2);
        }
    }
    else
    {
        /* output is stereo */
        if(ch1==2){
            /* input is stereo */
            sep_stereo(  in,    work1, work2, n1    );
            change_time( work1, in,    n1,    n2    );
            change_time( work2, work1, n1,    n2    );
            mix_stereo(  in,    work1, out,   n2    );
        }
        else
        {
            /* input is mono */
            change_time(in,work1,n1,n2);
            make_stereo(work1,out,  n2);
        }
    }
} /* convert_audio */
